'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var client = require('@apollo/client');
var fetch = _interopDefault(require('isomorphic-unfetch'));
var testing = require('@apollo/client/testing');
var graphql = require('graphql');
var graphqlTools = require('graphql-tools');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var fetchGraphQLSchema = function fetchGraphQLSchema(url, readable, init) {
  if (readable === void 0) {
    readable = true;
  }

  try {
    var headers = init ? init.headers : {};
    return Promise.resolve(fetch(url, _extends({
      method: 'POST',
      headers: _extends({
        Accept: 'application/json',
        'Content-Type': 'application/json'
      }, headers),
      body: JSON.stringify({
        query: graphql.getIntrospectionQuery
      })
    }, init)).then(function (res) {
      return res.json();
    }).then(function (schemaJSON) {
      if (readable) {
        return graphql.printSchema(graphql.buildClientSchema(schemaJSON.data));
      }

      return JSON.stringify(schemaJSON, null, 2);
    }));
  } catch (e) {
    return Promise.reject(e);
  }
};

function delay(ms) {
  return new Promise(function (resolve) {
    setTimeout(function () {
      resolve();
    }, ms);
  });
}

function createMockLink(schema, rootValue, context, options) {
  if (rootValue === void 0) {
    rootValue = {};
  }

  if (context === void 0) {
    context = {};
  }

  if (options === void 0) {
    options = {};
  }

  var delayMs = options && options.delay || 200;
  return new client.ApolloLink(function (operation) {
    return new client.Observable(function (observer) {
      var query = operation.query,
          operationName = operation.operationName,
          variables = operation.variables;
      delay(delayMs).then(function () {
        return graphql.graphql(schema, graphql.print(query), rootValue, context, variables, operationName);
      }).then(function (result) {
        var _result$errors;

        var originalError = result === null || result === void 0 ? void 0 : (_result$errors = result.errors) === null || _result$errors === void 0 ? void 0 : _result$errors[0].originalError;

        if (originalError) {
          // @ts-ignore
          var graphQLErrors = originalError === null || originalError === void 0 ? void 0 : originalError.graphQLErrors;

          if (graphQLErrors) {
            observer.next({
              errors: graphQLErrors
            });
          } // @ts-ignore


          var networkError = originalError.networkError;

          if (networkError) {
            observer.error(networkError);
          }
        } else {
          observer.next(result);
        }

        observer.complete();
      })["catch"](observer.error.bind(observer));
    });
  });
}

function createErrorLink(graphQLError) {
  return new client.ApolloLink(function () {
    return new client.Observable(function (observer) {
      delay(100).then(function () {
        observer.next({
          errors: createGraphQLErrorMessage(graphQLError)
        });
        observer.complete();
      })["catch"](observer.error.bind(observer));
    });
  });
}
function createLoadingLink() {
  return new client.ApolloLink(function () {
    return new client.Observable(function () {});
  });
}
function createApolloClient(_ref) {
  var mocks = _ref.mocks,
      _ref$cacheOptions = _ref.cacheOptions,
      cacheOptions = _ref$cacheOptions === void 0 ? {} : _ref$cacheOptions,
      _ref$clientOptions = _ref.clientOptions,
      clientOptions = _ref$clientOptions === void 0 ? {} : _ref$clientOptions,
      _ref$links = _ref.links,
      links = _ref$links === void 0 ? function () {
    return [];
  } : _ref$links,
      _ref$addTypename = _ref.addTypename,
      addTypename = _ref$addTypename === void 0 ? false : _ref$addTypename;
  var mockLink;

  if (!Array.isArray(mocks)) {
    var apolloLinkOptions = {};
    var resolvers = mocks.resolvers,
        introspectionResult = mocks.introspectionResult,
        typeResolvers = mocks.typeResolvers,
        rootValue = mocks.rootValue,
        context = mocks.context,
        _delay = mocks.delay;
    var schema = graphql.buildClientSchema(introspectionResult);
    var mockOptions = {
      schema: schema
    };
    mockOptions = _extends({}, mockOptions, {
      mocks: resolvers
    });
    graphqlTools.addMockFunctionsToSchema(mockOptions);

    if (!!typeResolvers) {
      graphqlTools.addResolveFunctionsToSchema({
        schema: schema,
        resolvers: typeResolvers
      });
    }

    if (_delay) {
      apolloLinkOptions.delay = _delay;
    }

    mockLink = createMockLink(schema, rootValue, context, apolloLinkOptions);
  } else {
    mockLink = new testing.MockLink(mocks, addTypename);
  }

  var cache = new client.InMemoryCache(_extends({}, cacheOptions, {
    addTypename: addTypename
  }));
  return new client.ApolloClient(_extends({
    cache: cache,
    link: client.ApolloLink.from([].concat(links(cache), [mockLink]))
  }, clientOptions));
}
function createGraphQLErrorMessage(graphQLError) {
  if (graphQLError) {
    return typeof graphQLError === 'string' ? [new graphql.GraphQLError(graphQLError)] : graphQLError;
  }

  return [new graphql.GraphQLError('Unspecified error from ErrorProvider.')];
}
function createMocks(query, _ref2) {
  var variables = _ref2.variables,
      data = _ref2.data,
      newData = _ref2.newData,
      graphqlErrors = _ref2.graphqlErrors,
      error = _ref2.error,
      _ref2$delay = _ref2.delay,
      delay = _ref2$delay === void 0 ? 200 : _ref2$delay;
  return [_extends({
    request: {
      query: query,
      variables: variables
    }
  }, !error ? {
    result: _extends({
      data: data
    }, graphqlErrors ? {
      errors: createGraphQLErrorMessage(graphqlErrors)
    } : {})
  } : {}, {
    error: error,
    newData: newData,
    delay: delay
  })];
}

var ApolloMockedProvider = function ApolloMockedProvider(_ref) {
  var children = _ref.children,
      _ref$Provider = _ref.Provider,
      Provider = _ref$Provider === void 0 ? client.ApolloProvider : _ref$Provider,
      rest = _objectWithoutPropertiesLoose(_ref, ["children", "Provider"]);

  var client$1 = createApolloClient(rest);
  return React.createElement(Provider, {
    client: client$1
  }, children);
};

var ApolloLoadingProvider = function ApolloLoadingProvider(_ref) {
  var _ref$Provider = _ref.Provider,
      Provider = _ref$Provider === void 0 ? client.ApolloProvider : _ref$Provider,
      children = _ref.children;
  var client$1 = new client.ApolloClient({
    link: createLoadingLink(),
    cache: new client.InMemoryCache()
  });
  return React.createElement(Provider, {
    client: client$1
  }, children);
};

var ApolloErrorProvider = function ApolloErrorProvider(_ref) {
  var children = _ref.children,
      graphQLError = _ref.graphQLError,
      _ref$Provider = _ref.Provider,
      Provider = _ref$Provider === void 0 ? client.ApolloProvider : _ref$Provider;
  var client$1 = new client.ApolloClient({
    link: createErrorLink(graphQLError),
    cache: new client.InMemoryCache()
  });
  return React.createElement(Provider, {
    client: client$1
  }, children);
};

exports.ApolloErrorProvider = ApolloErrorProvider;
exports.ApolloLoadingProvider = ApolloLoadingProvider;
exports.ApolloMockedProvider = ApolloMockedProvider;
exports.createApolloClient = createApolloClient;
exports.createErrorLink = createErrorLink;
exports.createGraphQLErrorMessage = createGraphQLErrorMessage;
exports.createLoadingLink = createLoadingLink;
exports.createMocks = createMocks;
exports.fetchGraphQLSchema = fetchGraphQLSchema;
//# sourceMappingURL=apollo-mocked.cjs.development.js.map
